@@ -212,7 +212,7 @@ fjoeders=function(u,v,cpar,du = FALSE){
     pdfu = ccond*(1-cpar)*(1-(1-ud)*vd/ctem2)/(1-v)
   }
   out = cbind(ccond,pdf,ccond1,pdf1,pdfu)
-  out[u==0,3] = 0
+  out[u<1e-10,3] = 0
   out[v==1,] = 0
   out[u==1,2:5] = 0
   out
@@ -232,6 +232,9 @@ fjoeders=function(u,v,cpar,du = FALSE){
 #' @export
 #'
 fmtcjders=function(u,v,cpar,du = FALSE){
+
+    cpar[cpar < 1e-6] = 1e-6
+    cpar[cpar >40] = 40
     ud=u^(-cpar);
     vd=v^(-cpar);
     cpar2=1/cpar/cpar;
@@ -253,10 +256,10 @@ fmtcjders=function(u,v,cpar,du = FALSE){
     pdfu=0;
     if(du==TRUE) pdfu = pdf*(-cpar-1+(2*cpar+1)*ud/(ud+vd-1))/u
     if(du==FALSE) pdfu = (cpar+1)*ccond*(-1/v + vd1/uvd)
-    
+
 
     out = cbind(ccond,ccond1,pdf,pdf1,pdfu)
-    out[u==0,1:5] = 0
+    out[u<1e-10,1:5] = 0
     out[v==0,2:5] = 0
     out[v==0,1] = 1
     out
@@ -277,6 +280,8 @@ fmtcjders=function(u,v,cpar,du = FALSE){
 #' @export
 #'
 fgumders=function(u,v,cpar,du = FALSE){
+
+  cpar[cpar<1+1e-6]=1+1e-6
   ul=-log(u);
   uul=u*ul;
   vl=-log(v);
@@ -313,7 +318,7 @@ fgumders=function(u,v,cpar,du = FALSE){
     cf2 = -1/v-(cpar-1)/v/vl
     pdfu = ccond*(l1v+cf1+cf2)
   }
-  
+
   tem1=(ut*ull+vt*vll)/uvt;
   tem2=-luvt/cpar/cpar;
   uvd1=uvd*(tem2+tem1/cpar);
@@ -321,7 +326,7 @@ fgumders=function(u,v,cpar,du = FALSE){
   ccond1=ccond*(-uvd1+vll+tem12);
   pdf1=-pdf*uvd1+l1cdf*(ull+vll+2*tem12)+l2cdf*(1/(cpar-1)+ull+vll+tem12-tem1);
   out=cbind(ccond, ccond1, pdf, pdf1, pdfu)
-  out[u==0,1:5] = 0
+  out[u<1e-10,1:5] = 0
   out[u==1,2:5] = 0
   out[u==1,1] = 1
   out[v==0,2:5] = 0
@@ -346,8 +351,8 @@ fgumders=function(u,v,cpar,du = FALSE){
 #' @export
 #'
 ffrkders=function(u,v,cpar,du = FALSE){
-  ind0 = (cpar==0)
-  ind1 = (cpar!=0)
+  ind0 = (abs(cpar)<1e-5)
+  ind1 = (abs(cpar)>=1e-5)
   n=length(u)
   ccond=rep(0,n)
   pdf = ccond
@@ -385,7 +390,7 @@ ffrkders=function(u,v,cpar,du = FALSE){
     pdfu[ind1] = 0;
     if(du==TRUE) pdfu[ind1] = cpar*pdf[ind1]*(2*ue*(1-ve)/den-1)
     if(du==FALSE) pdfu[ind1] = -cpar*ccond[ind1]*(1-ccond[ind1])
-    
+
 
   }
   cbind(ccond, ccond1, pdf, pdf1, pdfu)
@@ -413,7 +418,7 @@ ffgmders=function(u,v,cpar,du = FALSE){
   pdfu=0;
   if(du==TRUE) pdfu = -2*cpar*(1-2*v)
   if(du==FALSE) pdfu = -2*cpar*u*(1-u)
-  
+
   cbind(ccond, ccond1, pdf, pdf1, pdfu)
 }
 
@@ -456,7 +461,7 @@ fnorders=function(u,v,cpar,du = FALSE){
 
 
   out=cbind(ccond, ccond1, pdf, pdf1, pdfu)
-  out[u==0,2:5] = 0
+  out[u<1e-10,2:5] = 0
   out[u==1,2:5] = 0
   out[v==0,2:5] = 0
   out[v==1,2:5] = 0
@@ -503,9 +508,9 @@ ftdersP=function(u,v,cpar,dfC,du = FALSE){
   }
   out=cbind(ccond, ccond1, pdf, pdf1, pdfu)
   cf1 = sqrt(rho2/(dfC+1))
-  out[u==0,2:4] = 0
+  out[u<1e-10,2:4] = 0
   out[u==1,2:4] = 0
-  out[u==0,5] = Inf
+  out[u<1e-10,5] = Inf
   out[u==1,5] = -Inf
   out[v==0,3:5] = 0
   out[v==1,3:5] = 0
@@ -554,7 +559,7 @@ ftders=function(u,v,cpar,nu,du = FALSE){
     pdfu = pdf*wu*(x*sig*(nu+1)/(nu+x^2) - w*(nu+2)/(nu+1+w^2)) }
 
   out=cbind(ccond, ccond1, pdf, pdf1, pdfu)
-  out[u==0,2:5] = 0
+  out[u<1e-10,2:5] = 0
   out[u==1,2:5] = 1
   out
 }
